% This script creates the reference state vector according to the
% propagation equation in the article
%
% Julien LESOUPLE 12 sep 2018

%Parameters
delta_t = 0.5; % Sampling period
sigma_a=2; % User acceleration standard deviation
sigma_b=sqrt(0.5*2*1e-19); % User clock bias standard deviation
sigma_d=sqrt(2*pi^2*2*1e-20); % User clock drift variance
c=299792458; % Light speed
T=200; % Number of samples

%Initilialisation
X=[zeros(6,1);400;0];% Parameters
Xt=zeros(8,200);
Xt(:,1)=X;

%Propagation matrix
F1=[1 delta_t ; 0 1]; % Bloc matrix for propagation
F=kron(eye(4),F1); % Propagation matrix

%State noise covariance matrix
Qx=sigma_a^2*[delta_t^3/3 delta_t^2/2 ; delta_t^2/2 delta_t];
Qb=c^2*[(sigma_b^2*delta_t + sigma_d^2*delta_t^3/3) sigma_d^2*delta_t^2/2 ; sigma_d^2*delta_t^2/2 sigma_d^2*delta_t];
Q=blkdiag(kron(eye(3),Qx),Qb);

%Position
for t=1:T-1
    Xt(:,t+1)=F*Xt(:,t)+mvnrnd(zeros(8,1),Q)';
end