% This script performs the two filtering stratgies presented in the paper :
% the standard EKF and the EKF with sparse additive bias estimation
% 
% Julien LESOUPLE 12 sep 2018

% Parameters
elev_mask=5*pi/180; % Elevation threshold for weighting
lambda=1; % Regularization parameter
mu=1; % Smoothing parameter

% Initialize
X_EKF=zeros(8,T);
X_sparse=zeros(8,T);
X_sparse_smooth=zeros(8,T);
mt_hat_sparse=zeros(2*n,T);
mt_hat_sparse_smooth=zeros(2*n,T);

t=1;
X_EKF(:,t)=(H'*H)\H'*yt(:,t);
X_sparse(:,t)=X_EKF(:,t);
X_sparse_smooth(:,t)=X_EKF(:,t);

error_ECEF_Cov_EKF=sigma_uere^2*eye(4)/(H1'*H1);
error_xyzh_EKF=diag(error_ECEF_Cov_EKF);

P_EKF=blkdiag(error_xyzh_EKF(1), 0.01, error_xyzh_EKF(2), 0.01, ...
    error_xyzh_EKF(3), 0.01, error_xyzh_EKF(4), 1);

for t=2:T
    % Prediction
    X_EKF(:,t)=F*X_EKF(:,t-1);
    X_sparse(:,t)=F*X_sparse(:,t-1);
    X_sparse_smooth(:,t)=F*X_sparse_smooth(:,t-1);
    
    P_EKF=F*P_EKF*F+Q;
    
    % Correct measurements here

    weights_c=weights_cn0(CN0(:,t));
    weights_etemp=ones(n,1);
    weights_etemp(elev<elev_mask)=(sin(elev(elev<elev_mask))/sin(elev_mask)).^2;
    weights_e=[weights_etemp;weights_etemp];
    weights=weights_c.*weights_e;
    
    m_pr_sparse = sparse_gnss( yt(1:n,t),H1, lambda, weights(1:n));
    m_dop_sparse = sparse_gnss( yt(n+1:2*n,t),H1,lambda, weights(n+1:2*n));
    
    m_sparse =[m_pr_sparse;m_dop_sparse];
    
    Delta_Y_sparse=yt(:,t)-m_sparse;
    
    mt_hat_sparse(:,t)=m_sparse;
    
    m_pr_sparse_smooth = sparse_smooth_gnss( yt(1:n,t),H1, mt_hat_sparse_smooth(1:n,t-1),lambda, mu, weights(1:n));
    m_dop_sparse_smooth = sparse_smooth_gnss( yt(n+1:2*n,t), H1, mt_hat_sparse_smooth(n+1:2*n,t-1),lambda, mu, weights(n+1:2*n));
    
    m_sparse_smooth =[m_pr_sparse_smooth;m_dop_sparse_smooth];
    
    Delta_Y_sparse_smooth=yt(:,t)-m_sparse_smooth;
    
    mt_hat_sparse_smooth(:,t)=m_sparse_smooth;
    
    %% Kalman Filter
    % Matrix for Square root Kalman
    R_sqrt=sqrtm(R);
    Q_sqrt=sqrtm(Q);
    P_sqrt=sqrtm(P_EKF);
    
    big_mat_kal=[R_sqrt, H*F*P_sqrt,H*Q_sqrt ;zeros(8,2*n), F*P_sqrt, Q_sqrt]';
    [~,Rbmk]=qr(big_mat_kal);
    data_sqrt_kal_mat=Rbmk';
    S_sqrt=data_sqrt_kal_mat(1:2*n,1:2*n);
    KS_sqrt=data_sqrt_kal_mat(2*n+1:end,1:2*n);
    P_sqrt=data_sqrt_kal_mat(2*n+1:end,2*n+1:2*n+8);
    delta_x_EKF=KS_sqrt/S_sqrt*(yt(:,t)-H*X_EKF(:,t));
    delta_x_sparse=KS_sqrt/S_sqrt*(Delta_Y_sparse-H*X_sparse(:,t));
    delta_x_sparse_smooth=KS_sqrt/S_sqrt*(Delta_Y_sparse_smooth-H*X_sparse_smooth(:,t));
    
    X_EKF(:,t)=X_EKF(:,t)+delta_x_EKF;
    X_sparse(:,t)=X_sparse(:,t)+delta_x_sparse;
    X_sparse_smooth(:,t)=X_sparse_smooth(:,t)+delta_x_sparse_smooth;
    
    
    P_EKF=P_sqrt*P_sqrt';
    
    
end
