function [ m, n_iter ] = sparse_smooth_gnss( y,H, m_ante, lambda, mu, weights)
%SPARSE_SMOOTH_GNSS Solves weigthted-GNSS Sparse estimation with smoothing
%   Solves argmin_m 1/2||y-Hx-m||_2^2+lambda*||W*m||_1+mu*||W*(m-m_ante)||_1
%   y is the vector of measurements (Pseudoranges and/or Pseudorange rates)
%   H is the mixing matrix of the navigation equation
%   lambda is the hyperparameter of the relaxation
%   weights is the vector of diagonal element of W. If not provided,
%   default is 1
%
% Julien LESOUPLE 12 sep 2018

if nargin==3
    weights=ones(length(y),1);
end

if sum(weights<0)>0
    error('weights must be positive')
end
n=length(y);

% Pseudo-inverse via Cholesky
R=chol(H'*H);
P=H*(R\(R'\H'));

% Reformulation to LASSO
y_tilde=(eye(n)-P)*y;
H_tilde=(eye(n)-P);

% Solve LASSO
z=solveLassoSmooth(y_tilde,H_tilde,weights, m_ante, lambda, mu);

theta=z.beta;
n_iter=z.nb_iter;

% Finally deduce bias vector
m=theta;

end

